// Apogee.cpp : implementation file
//

#include "stdafx.h"
#include "Apogee.h"
#include <afx.h>
#include <stdlib.h>

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CApogee

CApogee::CApogee()
{
	AltaCamera = NULL;
	ExposureTime=10;
	Height=512;
	Width=512;
	hBin=1;
	vBin=1;
	xMax=1024;
	xMin=0;
	yMin=0;
	yMax=1024;
	Time=10;
	Temperature=20;
	FlushWait=0;
	PixelSizeX=0;
	PixelSizeY=0;
}

CApogee::~CApogee()
{
	AltaCamera = NULL;
}


BEGIN_MESSAGE_MAP(CApogee, CWnd)
	//{{AFX_MSG_MAP(CApogee)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// CApogee message handlers

bool CApogee::InitCamera()
{		
	HRESULT hr;	
	hr=AltaCamera.CreateInstance(__uuidof(Camera2));
	if (FAILED(hr)) return false;	
	 hr=AltaCamera->Init(Apn_Interface_USB,0,0,0x0);	
	if (FAILED(hr)) {
		ICamDiscoverPtr discover;
		hr=discover.CreateInstance(__uuidof(CamDiscover));
		if (FAILED(hr)) return false;
		discover->DlgCheckEthernet=true;
		discover->ShowDialog(true);
		if (!discover->ValidSelection) return false;
		hr=AltaCamera->Init(discover->SelectedInterface,
			discover->SelectedCamIdOne,
			discover->SelectedCamIdTwo,
			0x0);
		if (FAILED(hr)) return false;
	}
	Width=AltaCamera->GetPhysicalColumns();
	Height=AltaCamera->GetPhysicalRows();
	PixelSizeX=AltaCamera->GetPixelSizeX();
	PixelSizeY=AltaCamera->GetPixelSizeY();
	AltaCamera->PutCoolerEnable(true);	
	Temperature=-30;//(long)AltaCamera->GetTempCCD();
	ResetSystem();
	return true;
}

void CApogee::SetParameters(int ahBin, int avBin, int axMin,int axMax,int ayMin,int ayMax,int aTime,int aTemperature, bool aExternalTrigger,unsigned int aFlushWait) {	     
	hBin=ahBin;
	vBin=avBin;
	xMax=axMax;
	xMin=axMin;
	yMin=ayMin;
	yMax=ayMax;	
	FlushWait=aFlushWait;
	Temperature=aTemperature;
	ExternalTrigger=aExternalTrigger;
	ExposureTime=aTime;
	TransferParameters();
}

void CApogee::TransferParameters() {
	if (xMax>Width) xMax=Width;
	if (yMax>Height) yMax=Height;
	AltaCamera->DigitizeOverscan = false;
	AltaCamera->PutRoiStartX(xMin);
	AltaCamera->PutRoiStartY(yMin);
	AltaCamera->PutRoiPixelsH((xMax-xMin+1)/hBin);
	AltaCamera->PutRoiPixelsV((yMax-yMin+1)/vBin);
	AltaCamera->PutRoiBinningH(hBin);
	AltaCamera->PutRoiBinningV(vBin);
	AltaCamera->PutCoolerEnable(true);
	AltaCamera->PutCoolerSetPoint(Temperature);
	if (ExternalTrigger) AltaCamera->PutCameraMode(Apn_CameraMode_ExternalTrigger);
	else AltaCamera->PutCameraMode(Apn_CameraMode_Normal);
	AltaCamera->PutDisableShutter(true);
	AltaCamera->PutForceShutterOpen(true);
	AltaCamera->PutFanMode(Apn_FanMode_Off);
}

bool CApogee::TakePicture(int NrImages,unsigned short* Image[]) {	
	for(int i=0;i<NrImages;i++) {				
		//We need to reset the system after each image
		//if not it can happen that the camera stalls after having taken an image which uses binning		
		//ResetSystem();
		/*HRESULT hr;	
		AltaCamera = NULL;
		hr=AltaCamera.CreateInstance(__uuidof(Camera2));
		if (FAILED(hr)) return false;	
		hr=AltaCamera->Init(Apn_Interface_USB,0,0,0x0);
		if (FAILED(hr)) {
		  AfxMessageBox("CApogee::TakePicture : camera reinitialization failed");					 
		} else ResetSystem();*/
		ResetSystem();
		//AltaCamera->PutDisableShutter(true);				
		//AltaCamera->PutForceShutterOpen(true);
		AltaCamera->Expose(ExposureTime*0.001,true);
		
		DWORD start=GetTickCount();
		int status=99;		
		while ((status!=3) && (status>0)  && ((GetTickCount()-start)<60000)) {
			status=AltaCamera->GetImagingStatus();		
		}
		if (status!=3) {
			CString buf;
			buf.Format("CApogee::TakePicture : camera not delivering image, error %i",status);
			AfxMessageBox(buf);
			if (status<0) {
				HRESULT hr;	
				 hr=AltaCamera->Init(Apn_Interface_USB,0,0,0x0);
				 if (FAILED(hr)) {
					 AfxMessageBox("CApogee::TakePicture : camera reinitialization failed");					 
				 } else ResetSystem();
			}
			return false;	
		}
		AltaCamera->GetImage((long)Image[i]);
		//AltaCamera->PutForceShutterOpen(false);
		if ((i<(NrImages-1)) && (FlushWait>0)) {
			DWORD Start=GetTickCount();
			while ((GetTickCount()-Start)<FlushWait);
		}
	}

	//TestImage
	/*int width=xMax-xMin;
	for(long i=0;i<NrImages;i++) {		
		for (unsigned short x=xMin;x<xMax;x++)
			for (unsigned short y=yMin;y<yMax;y++)
				Image[i][x+y*width]=(unsigned short)(128*(x+y));
	}*/
	return true;
}

void CApogee::TemperatureDialog()
{
	AltaCamera->ShowTempDialog();
}

double CApogee::GetCCDTemperature()
{
	return AltaCamera->GetTempCCD();
}

void CApogee::LEDDialog()
{
	AltaCamera->ShowLedDialog();
}

void CApogee::IODialog()
{
	AltaCamera->ShowIoDialog();
}

void CApogee::DiscoverCameraDialog()
{
	ICamDiscoverPtr discover;
	HRESULT hr;
	hr=discover.CreateInstance(__uuidof(CamDiscover));
	if (FAILED(hr)) return;
	discover->ShowDialog(true);
}

void CApogee::TestTriggerLine()
{
	AltaCamera->PutIoPortAssignment(0);
	AltaCamera->PutIoPortDirection(0x3F);
	for (int i=0;i<5;i++) {
		AltaCamera->PutIoPortData(0);
		DWORD Start=GetTickCount();
		while ((GetTickCount()-Start)<1000);
		AltaCamera->PutIoPortData(0x01);//0x3F);
		Start=GetTickCount();
		while ((GetTickCount()-Start)<1000);		
	}
	AltaCamera->PutIoPortAssignment(0x3F);
}

void CApogee::ResetSystem()
{	
	AltaCamera->ResetSystem();
	TransferParameters();
	AltaCamera->DigitizeOverscan = false;
	AltaCamera->PutCoolerEnable(true);	
	AltaCamera->PutCoolerSetPoint(Temperature);
	//AltaCamera->PutLedMode(Apn_LedMode_DisableAll);
	AltaCamera->PutLedMode(Apn_LedMode_EnableAll);
	AltaCamera->PutLedA(Apn_LedState_ImageActive);
	AltaCamera->PutLedB(Apn_LedState_ExtTriggerWaiting);
	AltaCamera->PutIoPortAssignment(0x3F);	
	AltaCamera->PutIoPortDirection(0);	
	AltaCamera->PutDataBits(Apn_Resolution_SixteenBit);	
	//AltaCamera->PutDataBits(Apn_Resolution_TwelveBit);	
	//long gain=AltaCamera->GetGainTwelveBit();
	//AltaCamera->PutGainTwelveBit(600);
//	AltaCamera->PutCameraMode(Apn_CameraMode_Normal);
//	AltaCamera->PutCameraMode(Apn_CameraMode_ExternalShutter);
	//AltaCamera->PutDisableShutter(true);	
}
