// ApogeeServer.cpp : Defines the class behaviors for the application.
//

#include "stdafx.h"
#include "ApogeeServer.h"
#include "ApogeeServerDlg.h"
#include "TCPIPServer.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CApogeeServerApp

BEGIN_MESSAGE_MAP(CApogeeServerApp, CWinApp)
	//{{AFX_MSG_MAP(CApogeeServerApp)
	//}}AFX_MSG
	ON_COMMAND(ID_HELP, CWinApp::OnHelp)
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CApogeeServerApp construction

CApogeeServerApp::CApogeeServerApp()
{
	dlg=NULL;
}

/////////////////////////////////////////////////////////////////////////////
// The one and only CApogeeServerApp object

CApogeeServerApp ApogeeServerApp;
CTCPIPServer *TCPIPServer;


#define MAX_LOADSTRING 100
static TCHAR szMutexName[MAX_LOADSTRING];              // Mutex name

/////////////////////////////////////////////////////////////////////////////
// Creates a mutex only for one application
//    This function tries to create a mutex, but if specified mutex exits
//    this function returns NULL.
HANDLE CreateOneAppMutex(LPCTSTR lpName)
{
    HANDLE hMutex;
    hMutex = CreateMutex(NULL, TRUE, lpName);   // Create mutex
    switch(GetLastError())
    {
    case ERROR_SUCCESS:
        // Mutex created successfully. There is no instances running
        break;
    case ERROR_ALREADY_EXISTS:
        // Mutex already exists so there is a running instance of our app.
        hMutex = NULL;
        break;
    default:
        // Failed to create mutex by unknown reason
        break;
    }
    return hMutex;
}


/////////////////////////////////////////////////////////////////////////////
// CApogeeServerApp initialization

BOOL CApogeeServerApp::InitInstance()
{
	LastActiveTickCount=GetTickCount();
	LoadString(m_hInstance, IDC_MUTEX, szMutexName, MAX_LOADSTRING);
	Mutex=CreateOneAppMutex(szMutexName);
	if (Mutex==NULL) {
		//another copy of Control is already running
		//->disable hardware access
		AfxMessageBox("CApogeeServerApp::InitInstance: Another instance of Apogee server is running. Disabling hardware access. ");		
		HardwareAccess=false;
		return FALSE;
	} else HardwareAccess=true;		

	if (CoInitialize(NULL)!=S_OK) {
		AfxMessageBox("CApogeeServerApp::InitInstance : COM initialization failed");
	};
	/*if (!AfxSocketInit())
	{
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}*/

	// Initialize OLE libraries
	/*if (!AfxOleInit())
	{
		AfxMessageBox(IDP_OLE_INIT_FAILED);
		return FALSE;
	}
*/
	AfxEnableControlContainer();

	// Standard initialization

#ifdef _AFXDLL
	Enable3dControls();			// Call this when using MFC in a shared DLL
#else
	Enable3dControlsStatic();	// Call this when linking to MFC statically
#endif
	if (!AfxSocketInit()) {
		AfxMessageBox(IDP_SOCKETS_INIT_FAILED);
		return FALSE;
	}
/*	if (RunEmbedded() || RunAutomated())
	{
		COleTemplateServer::RegisterAll();
	}
	else
	{
		COleObjectFactory::UpdateRegistryAll();
	}*/
	
	Apogee=new CApogee();
	if (!Apogee->InitCamera()) {
		delete Apogee;
		AfxMessageBox("CApogeeServerApp::InitInstance : camera not initialized");
		return FALSE;
	}

	TCPIPServer=new CTCPIPServer();
	dlg=new CApogeeServerDlg(Apogee);	
	m_pMainWnd = dlg;
	dlg->SetCamera(Apogee);
	dlg->SetTCPIPServer(TCPIPServer);
	TCPIPServer->SetCamera(Apogee);
	TCPIPServer->SetApogeeServerDialog(dlg);
	TCPIPServer->StartNetworkServer(703);
	int nResponse;			
	do {
		nResponse=dlg->DoModal();
	} while (nResponse != IDCANCEL);
	TCPIPServer->StopNetworkServer();	
	delete TCPIPServer;
	delete dlg;
	dlg=NULL;

	if (Apogee) delete Apogee;
	CoUninitialize();
	// Since the dialog has been closed, return FALSE so that we exit the
	//  application, rather than start the application's message pump.
	return FALSE;
}

BOOL CApogeeServerApp::OnIdle(LONG lCount) 
{	
	//if no picture has been taken for five minutes (or after midnight) talk to camera to avoid it going to sleep
	if (((GetTickCount()-LastActiveTickCount)>(1000*60*5)) || ((GetTickCount()-LastActiveTickCount)<0)) {
		Apogee->ResetSystem();
		Apogee->TransferParameters();
	/*	Apogee->SetParameters(1,1,0,500,0,500,100,-20,false,0);
		const NrImages=1;
		unsigned short* Image[NrImages];	
		for (int i=0;i<NrImages;i++) Image[i]=new unsigned short[(1024*1024)/(1*1)];
		Apogee->TakePicture(NrImages,Image);	
		for (i=0;i<NrImages;i++) delete[] Image[i];		*/
		LastActiveTickCount=GetTickCount();
	}
	CWinApp::OnIdle(lCount);
	return true;
}

void CApogeeServerApp::WakeUp()
{
	LastActiveTickCount=GetTickCount();
}
