// TCPIPServer.cpp: implementation of the CTCPIPServer class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "ApogeeServer.h"
#include "TCPIPServer.h"
#include "clntsock.h"
#include "lstnsock.h"


//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

CTCPIPServer::CTCPIPServer() 
:CNetworkServer() {		
	hBin=1;
	vBin=1;
	xMax=512;
	xMin=1;
	yMin=1;
	yMax=512;
	Time=10;
	Temperature=-30;
	LastTime=0;				
	HardwareAccess=true;	
	ApogeeServerDlg=NULL;
	LastCommand="";
	FlushWait=0;	
}

CTCPIPServer::~CTCPIPServer()
{
	
}

void CTCPIPServer::SendReady() {		
	CString buf;
	buf.Format("Ready");
	SendMsg(buf);	
}

void CTCPIPServer::SetParameters() {	
	bool ok;
	int error=0;
	ok=true;
	ok=ok && ReadInt(hBin);
	ok=ok && ReadInt(vBin);
	ok=ok && ReadInt(xMin);
	ok=ok && ReadInt(xMax);
	ok=ok && ReadInt(yMin);
	ok=ok && ReadInt(yMax);
	ok=ok && ReadInt(Time);
	ok=ok && ReadInt(Temperature);
	ok=ok && ReadInt(ExternalTrigger);	
	ok=ok && ReadInt(FlushWait);	
	if (xMax>Apogee->Width) {xMax=Apogee->Width;error=4;}
	if (yMax>Apogee->Height) {yMax=Apogee->Height;error=4;}
	if (xMin<0) {xMin=0;error=4;}
	if (yMin<0) {yMin=0;error=4;}
	if (hBin<=0) hBin=1;
	if (vBin<=0) vBin=1;
	if (hBin>Apogee->Width) hBin=Apogee->Width;
	if (vBin>Apogee->Height) vBin=Apogee->Height;
	Apogee->ResetSystem();
	Apogee->SetParameters(hBin,vBin,xMin,xMax,yMin,yMax,Time,Temperature,ExternalTrigger!=0,FlushWait);
}
	
void CTCPIPServer::TakeImage() {	
	long NrImages;
	bool ok;
	int error=0;
	ok=true;
	ok=ok && ReadInt(NrImages);		
	if (xMax>Apogee->Width) {xMax=Apogee->Width;error=4;}
	if (yMax>Apogee->Height) {yMax=Apogee->Height;error=5;}
	if (xMin<0) {xMin=0;error=6;}
	if (yMin<0) {yMin=0;error=7;}
	if (NrImages<0) {NrImages=1;error=8;}
	if (NrImages>30) {NrImages=30;error=9;}
	long size=(xMax-xMin+1)*(yMax-yMin+1)/(hBin*vBin);
	unsigned short* Image[30];
	for (int i=0;i<NrImages;i++) Image[i]=new unsigned short[size];
	if (ok) ok=Apogee->TakePicture(NrImages,Image);
	else error=1;
	unsigned short *RotatedImage;
	RotatedImage=new unsigned short[size];
	SendReady();
	for (i=0;i<NrImages;i++) {
		if (ok) ok=CheckReady(); else error=2;
		if (ok) {		
			int MyWidth=((xMax-xMin+1)/hBin);
			int MyHeight=((yMax-yMin+1)/vBin);
			if (MyWidth!=MyHeight) {
				//sorry, can't rotate
				WriteInteger(2*size);			
				SendData((unsigned char*)Image[i],size*2);			
			} else {
				//We need to rotate for our camera orientation
				for (int x=0;x<MyWidth;x++) {
					for (int y=0;y<MyHeight;y++)
						RotatedImage[x+y*MyWidth]=Image[i][y+x*MyWidth];
				}
				WriteInteger(2*size);			
				SendData((unsigned char*)RotatedImage,size*2);			
			}
		} else error=3;		
	} 
	delete[] RotatedImage;
	if (error!=0) {
		CString buf;
		buf.Format("CApogeeServerDlg::ProcessMessage(Image) : error %i",error);
		AfxMessageBox(buf);
	}
	for (i=0;i<NrImages;i++) delete Image[i];
}

void CTCPIPServer::ProcessMessage(CString Message) {
	((CApogeeServerApp*)AfxGetApp())->WakeUp();
	LastCommand=Message;
	UpdateStatus();
	if (Message=="Goodbye") StopServer();
	else if (Message=="Ready") SendReady();
	else if (Message=="Image") TakeImage();
	else if (Message=="Parameters") SetParameters();
	else {
		CString buf;
		buf.Format("CTCPIPServer::ProcessMessage : unknown message\n%s",Message);
		AfxMessageBox(buf);	
	}
	UpdateStatus();
}

void CTCPIPServer::StopServer() {
	DeleteClientSocket();
	if (DebugFile) (*DebugFile)<<"DeleteClientSocket"<<endl;
}

void CTCPIPServer::Disable()
{
	HardwareAccess=false;
}

void CTCPIPServer::SetCamera(CApogee *aApogee)
{
	Apogee=aApogee;
}

void CTCPIPServer::UpdateStatus()
{
	CString Status;
	double HeatsinkTemperature=0;
	double CCDTemperature=0;	
	double SizeX=0;
	double SizeY=0;
	if (Apogee) {
		if (Apogee->AltaCamera) {
			CCDTemperature=Apogee->GetCCDTemperature();
			HeatsinkTemperature=Apogee->AltaCamera->GetTempHeatsink();
			xMin=Apogee->AltaCamera->GetRoiStartX();
			yMin=Apogee->AltaCamera->GetRoiStartY();
			hBin=Apogee->AltaCamera->GetRoiBinningH();
			vBin=Apogee->AltaCamera->GetRoiBinningV();
			xMax=xMin+Apogee->AltaCamera->GetRoiPixelsH()*hBin-1;
			yMax=yMin+Apogee->AltaCamera->GetRoiPixelsV()*vBin-1;
			SizeX=Apogee->PixelSizeX;
			SizeY=Apogee->PixelSizeY;
		}
	}
	Status.Format("%s\nSet Temperature=%i\nTemperature=%.2f\nHeatsink Temp=%.2f\nxMin=%i xMax=%i xBin=%i SizeX=%.1f\nyMin=%i yMax=%i yBin=%i SizeY=%.1f\nExposure time=%i\nFlush wait=%i\n\nLast command=%s \nDebug %s", 
		(Connected) ? "Connected" : "Listening",Temperature,CCDTemperature,HeatsinkTemperature,xMin,xMax,hBin,SizeX,
		yMin,yMax,vBin,SizeY,Time,FlushWait,LastCommand,(DebugFile==NULL) ? "off" : "on");
	if (ApogeeServerDlg) ApogeeServerDlg->UpdateStatus(Status);
}

void CTCPIPServer::SetApogeeServerDialog(CApogeeServerDlg *aApogeeServerDlg)
{
	ApogeeServerDlg=aApogeeServerDlg;
}


